/*
 * Copyright (c) 2019, Mentor Graphics Corporation
 *
 * SPDX-License-Identifier: BSD-2-Clause
 */

#ifndef TA_MISS_TEST_H
#define TA_MISS_TEST_H

/* This UUID is generated with uuidgen */
#define TA_MISS_TEST_UUID { 0x528938ce, 0xfc59, 0x11e8, \
	{ 0x8e, 0xb2, 0xf2, 0x80, 0x1f, 0x1b, 0x9f, 0xd1 } }

#endif /* TA_MISS_TEST_H */
