/*
 * Copyright (c) 2019, Mentor Graphics Corporation
 *
 * SPDX-License-Identifier: BSD-2-Clause
 */

#ifndef USER_TA_HEADER_DEFINES_H
#define USER_TA_HEADER_DEFINES_H

#include <ta_miss_test.h>

#define TA_NAME		"MISS"
#define TA_UUID		TA_MISS_TEST_UUID

#define TA_FLAGS		(0UL)
#define TA_STACK_SIZE		(32 * 1024)
#define TA_DATA_SIZE		(32 * 1024)

#endif /* USER_TA_HEADER_DEFINES_H */
