/* SPDX-License-Identifier: BSD-2-Clause */
/*
 * Copyright (c) 2021 Open Mobile Platform LLC
 */

#ifndef USER_TA_HEADER_DEFINES_H
#define USER_TA_HEADER_DEFINES_H

#include <ta_supp_plugin.h>
#include <user_ta_header.h>

#define TA_UUID		TA_SUPP_PLUGIN_UUID
#define TA_FLAGS	0
#define TA_STACK_SIZE	(8 * 1024)
#define TA_DATA_SIZE	(32 * 1024)

#endif /* USER_TA_HEADER_DEFINES_H */
