/* SPDX-License-Identifier: BSD-3-Clause */
/*
 * Copyright (c) 2020, ARM Limited. All rights reserved.
 */

#ifndef USER_TA_HEADER_DEFINES_H
#define USER_TA_HEADER_DEFINES_H

#include <ta_tpm_log.h>

#define TA_UUID		TA_TPM_LOG_TEST_UUID

#define TA_FLAGS	0
#define TA_STACK_SIZE	(4 * 1024)
#define TA_DATA_SIZE	(32 * 1024)

#endif /* USER_TA_HEADER_DEFINES_H */
