/* SPDX-License-Identifier: BSD-2-Clause */
/*
 * Copyright (c) 2021 Open Mobile Platform LLC
 */

#ifndef TA_SUPP_PLUGIN_H
#define TA_SUPP_PLUGIN_H

#define TA_SUPP_PLUGIN_UUID { 0x380231ac, 0xfb99, 0x47ad, \
	{ 0xa6, 0x89, 0x9e, 0x01, 0x7e, 0xb6, 0xe7, 0x8a } }

#define TA_SUPP_PLUGIN_CMD_PASS_VALUES	0
#define TA_SUPP_PLUGIN_CMD_WRITE_ARR	1
#define TA_SUPP_PLUGIN_CMD_GET_ARR	2
#define TA_SUPP_PLUGIN_CMD_BAD_UUID	3
#define TA_SUPP_PLUGIN_CMD_BAD_IN_DATA	4
#define TA_SUPP_PLUGIN_CMD_BAD_IN_LEN	5
#define TA_SUPP_PLUGIN_CMD_UNKNOWN_UUID	6

#endif /* TA_SUPP_PLUGIN_H */
