/* SPDX-License-Identifier: BSD-2-Clause */
/*
 * Copyright (c) 2015, Linaro Limited
 * All rights reserved.
 */

#ifndef TA_CONCURRENT_LARGE_H
#define TA_CONCURRENT_LARGE_H

#include <ta_concurrent.h>

/* 5ce0c432-0ab0-40e5-a056-782ca0e6aba2 */
#define TA_CONCURRENT_LARGE_UUID { 0x5ce0c432, 0x0ab0, 0x40e5, \
	{ 0xa0,0x56, 0x78, 0x2c, 0xa0, 0xe6, 0xab, 0xa2 } }

#endif /*TA_CONCURRENT_LARGE_H */
