/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Copyright (c) 2023, Linaro Limited
 */

#ifndef XTEST_PKCS11_1000_H
#define XTEST_PKCS11_1000_H

/* Helper function for finding and detroying a token object */
int xtest_pkcs11_1028_destroy_token_object(void);

#endif /*XTEST_PKCS11_1000_H*/
