/* SPDX-License-Identifier: BSD-2-Clause */
/*
 * Copyright (c) 2021 Open Mobile Platform LLC
 */

#ifndef TEST_SUPP_PLUGIN_H
#define TEST_SUPP_PLUGIN_H

#define TEST_PLUGIN_UUID { 0xf07bfc66, 0x958c, 0x4a15, \
	{ 0x99, 0xc0, 0x26, 0x0e, 0x4e, 0x73, 0x75, 0xdd } }

#define TEST_PLUGIN_CMD_PING		0
#define TEST_PLUGIN_CMD_PASS_VALUES	1
#define TEST_PLUGIN_CMD_WRITE_ARR	2
#define TEST_PLUGIN_CMD_GET_ARR		3

#endif /* TEST_SUPP_PLUGIN_H */
