/* SPDX-License-Identifier: BSD-2-Clause */
/*
 * Copyright (c) 2014, Linaro Limited
 * All rights reserved.
 */
#ifndef ENC_FS_KEY_MANAGER_TEST_H
#define ENC_FS_KEY_MANAGER_TEST_H

/* This UUID is generated with uuidgen
   the ITU-T UUID generator at http://www.itu.int/ITU-T/asn1/uuid.html */
#define ENC_FS_KEY_MANAGER_TEST_UUID \
		{ 0x17E5E280, 0xD12E, 0x11E4,  \
			{ 0xA4, 0x1A, 0x00, 0x02, 0xA5, 0xD5, 0xC5, 0x1B } }
/* The TAFs ID implemented in this TA */
#define CMD_SELF_TESTS	0

#endif /*ENC_FS_KEY_MANAGER_TEST_H*/
